/*      Copyright (c) 1987 Shrier and Deihl. Licensed to Mix Software, Inc.
        Copyright (c) 1995 Mix Software, Inc. Multi-user version

        cbkey.c - CBT cbkey function
*/
#include <string.h>
#include "btree.h"

/*      Defined Functions
        cbkey                   copy current key into buffer
*/

/*      cbkey - copy current key into buffer

        Does not copy more than buflen bytes.
        Returns OK and copies key, I_BOI, I_EOI, or ERROR.
*/


I_EXPORT int I_ENTRY cbkey(Cbtree *tree, char *buffer, int buflen)
{
   Ctlrec *ctl;            /* control record for tree */
   int    len;
   int    status;

   SET_FC(FC_CBKEY)
   ctl = tree->ctl;
   if ((B_is_tree_valid(tree) == I_NO) || (B_is_ctl_valid(ctl) == I_NO))
      return (I_ERROR);
   if (tree->cur_key.status != I_OK) return (tree->cur_key.status);
   if (B_ctl_sync(ctl) != I_OK) return I_ERROR;
   status = B_key_sync(tree, &tree->cur_key);
   B_ctl_unlock(ctl);
   if (status == I_ERROR) {
      SET_CC(CC_SYNC)
      return (status);
      }
   if (tree->cur_key.status != I_OK) {
      /* current item was deleted and now at I_EOI */
      SET_EC(EC_DEL)
      return (tree->cur_key.status);
      }
   if (status != I_OK) SET_EC(EC_DEL)
   len = tree->cur_key.keylen;
   if (buflen < len) len = buflen;
   if (len > 0) memmove(buffer, tree->cur_key.key, len);
   B_ctl_unlock(ctl);
   return (I_OK);
   } /* cbkey */

/* end cbkey.c */
