/*      Copyright (c) 1994 Mix Software, Inc.

        ctlrec.h - include file for CBT control record routines
*/


#ifndef _CTLREC_H
#define _CTLREC_H

#include "bufpool.h"

/*
        The control record - always node zero.
        The Ctldisk plus the vparity is stored on the disk.
*/


typedef unsigned short Parity;  /* type of vertical parity element */

#define CBT_FILE_VERSION        300     /* file version number */
#if defined(__GNUC__)
   #define PACK __attribute__ ((packed))
#else
   #define PACK
#endif

typedef struct  ctldisk {       /* elements stored on disk */
        short   blksize PACK;        /* size of node block */
        Blk_Nbr root PACK;           /* block number of current root */
        Blk_Nbr highblock PACK;      /* next available block number */
        Blk_Nbr lowleaf PACK;        /* block number of lowest leaf */
        Blk_Nbr highleaf PACK;       /* block number of highest leaf */
        Blk_Nbr freechain PACK;      /* block number of free block chain */
        short   depth PACK;          /* depth of btree */
        Parity vparity PACK;         /* vertical parity check of all elements */
        unsigned short  ver PACK;    /* cbtree file version number */
        unsigned long   update PACK; /* shared file update count */
        long    keycount PACK;       /* number of keys in the tree */
} Ctldisk;

typedef struct  ctlrec {        /* entire control record in memory */
        Ctldisk disk;           /* elements read from disk */

        /* below elements exist only in the in-memory copy */
        struct  ctlrec  *next;  /* next in list of valid control records */
        int     nopen;          /* number of times this file is open */
        int     share;          /* shared/exclusive access flag */
        int     lock_state;     /* manual and auto file lock flags */
        Bfile   *bfile;         /* buffered file pointer */
        char    *fname;         /* buffer for name of file */
        unsigned long   update; /* local copy of shared file update count */
        unsigned read_lock;     /* offset of read lock byte */
        int     current;        /* 1 if file has been locked since last read */
        int     modified;       /* 1 if changed since last write */
} Ctlrec;

/*
        lock_state masks
*/

#define CTL_RLOCK               0x0003          /* read locking flags */
#define CTL_WLOCK               0x0030          /* write locking flags */
#define CTL_READ_LOCK           0x0002          /* locked for reading */
#define CTL_AUTO_READ_LOCK      0x0002          /* automatic lock for reading */
#define CTL_MANUAL_READ_LOCK    0x0003          /* manual lock for reading */
#define CTL_WRITE_LOCK          0x0020          /* locked for writing */
#define CTL_AUTO_WRITE_LOCK     0x0020          /* automatic write lock flag */
#define CTL_MANUAL_WRITE_LOCK   0x0030          /* manual write lock */

/*
        control record access routines
*/

Ctlrec  *B_ctlalloc(char *filename, int share);
Ctlrec  *B_ctlfind(char *filename);
int     B_ctlfree(Ctlrec *ctl);
int     B_ctlload(Ctlrec *ctl);
int     B_ctlsave(Ctlrec *ctl);
int     B_ctlmake(Ctlrec *ctl, int blksize);
int     B_is_ctl_valid(Ctlrec *ctl);
int     B_is_read_locked(Ctlrec *ctl);
int     B_is_write_locked(Ctlrec *ctl);
unsigned short  B_calc_parity(unsigned short *buf, int word_count);
int     B_ctl_sync(Ctlrec *ctl);
int     B_ctl_resync(Ctlrec *ctl, int update, int delay_sync);
int     B_ctl_unlock(Ctlrec *ctl);
int     B_ctl_unlock_all(Ctlrec *ctl);
int     B_read_lock(Ctlrec *ctl);
int     B_write_lock(Ctlrec *ctl);

#endif  /* _CTLREC_H */
/* end ctlrec.h */
