/*$no list*//*$no trace <<< cbtree.h >>> */
/*      Copyright 1987 Shrier and Deihl. Licensed to Mix Software, Inc.
        Copyright (c) 1994 Mix Software, multi-user version

        cbtree.h - application level declarations for CBT

        Application programs using the CBT library must include this
        header file in all source files.

        If your compiler does not support prototypes, see comments in
        cbtree.cfg.
*/
#include "cdtdef.h"

#ifndef _BTREE_H
/*
        The Cbtree index file identifier, returned by cbopen, is passed as
        an argument to most of the other routines to identify which index
        file is to be used.  It is rather like the FILE pointer used in
        the stdio library fopen routines.
*/
#define Cbtree  void          /* Cbtree index file identifier */
typedef unsigned long Item;   /* type for a key's associated item */
#endif

#ifndef CBTREE_SRC
   #ifndef ISAM_SRC
   #ifdef __POWERC
      #if __POWERC >= 200
         #pragma library "cbt*.lib"
      #endif
   #endif
   #endif
#endif

/*
        Declare all CBT user-level routines
*/
#define DEFAULT_COMPARE ((Cmp_Fn_Ptr) 0)

#ifdef __cplusplus
extern "C" {
#endif

/*
        define error code variables
*/

#if ! defined(_ERROR_VARS)
   #define _ERROR_VARS
   #if defined(__NT__) || defined(_WIN32) || defined(__WIN32__)
      #define cbconcode (cbget_concode())
      #define cbfcncode (cbget_fcncode())
      #define cberrcode (cbget_errcode())
   #else
      I_EXPORT_DATA int cbconcode;  /* context code - what it was doing */
      I_EXPORT_DATA int cbfcncode;  /* function code - who detected the error */
      I_EXPORT_DATA int cberrcode;  /* error code - the error itself */
   #endif
#endif

/* add buffers to the buffer pool */
I_EXPORT int I_ENTRY cbaddbufs(
            int nbrbufs);

/* return CBT file block size */
I_EXPORT int I_ENTRY cbblksize(
            char *filename);

/* return count of buffers */
I_EXPORT int I_ENTRY cbbufcnt(void);

/* return size of buffers */
I_EXPORT int I_ENTRY cbbufsize(void);

/* close an index file */
I_EXPORT int I_ENTRY cbclose(
            Cbtree *tree);

/* reset error codes */
I_EXPORT void I_ENTRY cbclear_error(void);

/* return text of cbconcode message */
I_EXPORT char * I_ENTRY cbconmsg(
            int con);

/* create and open a new index file */
I_EXPORT Cbtree * I_ENTRY cbcreate(
            char *filename,
            Cmp_Fn_Ptr keycmp,
            int blksize);

/* return item of current (first) key */
I_EXPORT int I_ENTRY cbcurr(
            Cbtree *tree,
            Item *item_ptr);

/* delete key/item */
I_EXPORT int I_ENTRY cbdelete(
            Cbtree *tree,
            char *key,
            int keylen,
            Item item);

/* return tree depth */
I_EXPORT int I_ENTRY cbdepth(
            Cbtree *tree);

/* return text of cberrcode message */
I_EXPORT char * I_ENTRY cberrmsg(
            int cbt_errno);

/* end processing, close open files */
I_EXPORT int I_ENTRY cbexit(void);

/* return text of cbfcncode message */
I_EXPORT char * I_ENTRY cbfcnmsg(
            int fcn);

/* return item of specified key */
I_EXPORT int I_ENTRY cbfind(
            Cbtree *tree,
            char *key,
            int keylen,
            Item *item);

/* return item of key/item match */
I_EXPORT int I_ENTRY cbfinditem(
            Cbtree *tree,
            char *key,
            int keylen,
            Item *item);

/* make marked key be the current key */
I_EXPORT int I_ENTRY cbfindmark(
            Cbtree *tree,
            Item *item_ptr);

/* flush all buffers to index file */
I_EXPORT int I_ENTRY cbflush(
            Cbtree *tree);

/* get error codes */
I_EXPORT int I_ENTRY cbget_concode(void);
I_EXPORT int I_ENTRY cbget_fcncode(void);
I_EXPORT int I_ENTRY cbget_errcode(void);

/* return item of first key */
I_EXPORT int I_ENTRY cbhead(
            Cbtree *tree,
            Item *item_ptr);

/* initialize processing, alloc buffers */
I_EXPORT int I_ENTRY cbinit(
            int bufcnt,
            int bufsize);
I_EXPORT int I_ENTRY cbinit_dos(
            int bufcnt,
            int bufsize);
I_EXPORT int I_ENTRY cbinit_mswin(
            int bufcnt,
            int bufsize);
I_EXPORT int I_ENTRY cbinit_os2(
            int bufcnt,
            int bufsize);

/* insert key/item */
I_EXPORT int I_ENTRY cbinsert(
            Cbtree *tree,
            char *key,
            int keylen,
            Item item,
            int dup);

/* copy current key into keybuf */
I_EXPORT int I_ENTRY cbkey(
            Cbtree *tree,
            char *buffer,
            int buflen);

/* compare two keys */
I_EXPORT int I_ENTRY cbkeycmp(
            char *keya,
            int lena,
            char *keyb,
            int lenb);

/* return length of current key */
I_EXPORT int I_ENTRY cbkeylen(
            Cbtree *tree,
            int *keylen);

/* lock the btree file */
I_EXPORT int I_ENTRY cblock(
            Cbtree *tree,
            int lock_type);

/* set retry count and timeout for locking */
I_EXPORT void I_ENTRY cblock_timeout(
            int tries,
            long delay,
            long timeout);

/* mark current key */
I_EXPORT int I_ENTRY cbmark(
            Cbtree *tree);

/* modify item and/or key */
I_EXPORT int I_ENTRY cbmodify(
            Cbtree *tree,
            char *oldkey,
            int oldkeylen,
            Item olditem,
            char *newkey,
            int newkeylen,
            Item newitem,
            int dup);

/* return item of next (first) key */
I_EXPORT int I_ENTRY cbnext(
            Cbtree *tree,
            Item *item_ptr);

/* open an existing index file */
I_EXPORT Cbtree * I_ENTRY cbopen(
            char *filename,
            Cmp_Fn_Ptr keycmp);
I_EXPORT Cbtree * I_ENTRY cbopen_shared(
            char *filename,
            Cmp_Fn_Ptr keycmp,
            int share,
            int readonly);

/* return item of previous (last) key */
I_EXPORT int I_ENTRY cbprev(
            Cbtree *tree,
            Item *item_ptr);

/* print message for current error */
I_EXPORT void I_ENTRY cbprterr(void);

/* remove buffers from the buffer pool */
I_EXPORT int I_ENTRY cbrmbufs(
            int nbrbufs);

/* create a new index file and open it for shared access */
I_EXPORT Cbtree * I_ENTRY cbscreate(
            char *filename,
            Cmp_Fn_Ptr keycmp,
            int blksize,
            int share);

/* set the print function used by cbprterr etc */
I_EXPORT Puts_Fn_Ptr I_ENTRY cbset_print(
            Puts_Fn_Ptr printfn);

/* return item of last key */
I_EXPORT int I_ENTRY cbtail(
            Cbtree *tree,
            Item *item_ptr);

/* unlock the file */
I_EXPORT int I_ENTRY cbunlock(
            Cbtree *tree,
            int lock_type);

/* set flags for flushing file buffers */
I_EXPORT void I_ENTRY cb_set_flush(
            int delay_sync,
            int flush_blks);

#ifdef __cplusplus
}
#endif
/* end cbtree.h */
/*$list*//*$trace       <<< cbtree.h >>> */
