/*      Copyright (c) 1987 Shrier and Deihl. Licensed to Mix Software, Inc.
        Copyright (c) 1995 Mix Software, Inc. Multi-user version

        matpre.c - ISAM INTERNAL match prefix of current record
*/
#include "isamlib.h"

/*
        I_mat_prefix - see if prefix of current record matches the number

        This function indicates if the current record is in a certain
                subtree.

        Returns I_YES (they match), I_NO (they don't), or I_ERROR.
*/

int I_mat_prefix(Cbtree *tree, int nbr)
   /* tree = current record is from this tree */
   /* nbr  = number to match to record's prefix */
{
   int status;
   char prefix;

   if ((status = cbkey(tree, &prefix, sizeof(prefix))) != I_OK) {
      if (status == I_ERROR) SET_EC(I_CBT)
      else SET_EC(I_NOCURRENT)
      return (I_ERROR);
      }
   return ((int) prefix == nbr ? I_YES : I_NO);
   } /* I_mat_prefix */

/*
        I_check_prefix - verify that the prefix on the current record
            matches the index.
        The position status of the index is updated if I_EOI or I_BOI.

        Returns I_OK (they match), I_EOI current prefix > index,
           I_BOI prefix < index or I_ERROR
*/

int I_check_prefix(Index_Obj *index)
{
   int status;
   char prefix;

   if ((status = cbkey(index->tree, &prefix, sizeof(prefix))) != I_OK) {
      if (status == I_ERROR) SET_EC(I_CBT)
      else if (status == I_BOI || status == I_EOI) {
         index->position_status = status;
         return status;
         }
      else if (status != I_FOUND && status != I_NOTFOUND) return (I_ERROR);
      else SET_EC(I_NOCURRENT)
      }
   if ((int) prefix == index->index_nbr) status = I_OK;
   else if ((int) prefix > index->index_nbr) status = I_EOI;
   else status = I_BOI;
   index->position_status = status;
   return status;
   } /* I_check_prefix */

